#include "stdafx.h"
#include "Cubes.h"
#include "CubesDoc.h"
#include "CubesView.h"
#include "VisualObject.h"
#include "InfoViewManager.h"
#include "log.h"

/**
 * ======== InfoViewManager::InfoViewManager ========
 *
 *  Purpose:
 *    Class constructor, sets default change units to 10%
 */

InfoViewManager::InfoViewManager() {
	transformIdx = 0;
	currentIdx = -1;
	internalMax = 0;

	// change units
	fade_units = 10;
	scale_units = 10;
	location_units = 10;

	xMin = 0;
	xMax = 0;
	yMin = 0;
	yMax = 0;
	zMin = 0;
	zMax = 0;

	currentProperty = NULL;
	clearCurrentGroup();
}

void InfoViewManager::clearCurrentGroup() {
	currentGroup = "__root__";
}

void InfoViewManager::Clear() {
	transformIdx = 0;
	currentIdx = -1;
	clearCurrentGroup();
}

void InfoViewManager::SetGroup() {
	if ( currentIdx != -1 ) {
		currentGroup = transformNameList[ currentIdx ];
		log_message1( "SetGroup SET CURRENT GROUP TO ", currentGroup );
	} else {
		log_message( "SetGroup DID NOT SET CURRENT GROUP" );
	}
}

IMFNodeObject* InfoViewManager::getGroup() {
	/*
	if ( strcmp( currentGroup, "__root__" ) != 0 ) {
		log_message1( "currentGroup is ", currentGroup );
		int groupIdx = getIdx( currentGroup );
		if ( groupIdx != -1 ) {
			log_message( "OK, I found it" );
			return( visualObjects[ groupIdx ] -> getGroup() );
		} else {
			log_message( "OK, I did not find it" );
		}
	}*/
	return( NULL );
}


/**
 * ======== InfoViewManager::Cycle ========
 *
 *  Purpose:  
 *    Cycle to the next object.
 */

void InfoViewManager::Cycle() {
	if ( transformIdx > 0 ) {
		currentIdx++;
		if ( currentIdx >= transformIdx ) {
			currentIdx = 0;
		}
		log_message1( "object cycled to: ", transformNameList[ currentIdx ] );
	}
}


/**
 * ======== InfoViewManager::FadeIn ========
 *
 *  Purpose:
 *    Decrease the transparency (i.e. make more "solid") of the current object.
 */

void InfoViewManager::FadeIn() {
	log_message1( "FadeIn called", transformNameList[ currentIdx ] );
	if (( currentIdx >= 0 ) && gotMaterial[ currentIdx ]) {
		ISFFloatObject* f = transparencyList[ currentIdx ];
		float value = 0.0f;
		f->get_Value(&value);
		value -= 0.01f * fade_units;
		if ( value < 0 ) {
			value = 0.0f;
		}
		f->put_Value(value);
	}
}



/**
 * ======== InfoViewManager::FadeOut ========
 *
 *  Purpose:
 *    Increase the transparency (i.e. make it more "see through") of the current object.
 */

void InfoViewManager::FadeOut() {
	log_message1( "FadeOut called", transformNameList[ currentIdx ] );
	if (( currentIdx >= 0 ) && gotMaterial[ currentIdx ]) {
		log_message( "GOT HERE" );
		ISFFloatObject* f = transparencyList[ currentIdx ];
		if ( f == NULL ) {
			log_message( "HEY, f is NULL " );
		}
		float value = 1.0f;
		f->get_Value(&value);
		value += 0.01f * fade_units;
		if ( value > 1.0f ) {
			value = 1.0f;
		}
		f->put_Value(value);
		log_message( "Successfully faded out " );
	}
}

void InfoViewManager::SetFadeUnits( char* strvalue ) {
	if ( strvalue != NULL ) {
		fade_units = atoi( strvalue );
	}
}
void InfoViewManager::SetScaleUnits( char* strvalue ) {
	if ( strvalue != NULL ) {
		scale_units = atoi( strvalue );
	}
}

void InfoViewManager::SetMax( char* maxstr ) {
	internalMax = atoi( maxstr );
}


/**
 *  ======== InfoViewManager::SetObject ========
 *
 *  Purpose:
 *    Generic method for assigning all subsequent commands to a specific object.
 *
 *  Input Parmaeters:
 *    name -- name of the object to associate all future commands with.
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetObject( char* name ) {
	currentIdx = getIdx( name );
	log_message1( "SetObject: ", name );
	log_messagei1( "..value: ", currentIdx );
}


/**
 *  ======== InfoViewManager::SetProperty ========
 *
 *  Purpose:
 *    Generic method for setting the type of property a subsequent "SetValue"
 *    method applies to.
 *
 *  Input Parameters:
 *    propertyName -- name of the property we are to associate with all future
 *      "SetValue" method calls.
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetProperty( char* propertyName ) {
	//log_message1( "SetProperty: ", propertyName );
	if ( currentProperty != NULL ) {
		free( currentProperty );
	}
	currentProperty = newstrcpy( propertyName );
}

void InfoViewManager::setXlocation( float value ) {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
		m_pTranslation->put_X( value );
		//log_messagef( "putting X location", value );
	}
}
void InfoViewManager::setYlocation( float value ) {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
		m_pTranslation->put_Y( value );
		//log_messagef( "putting X location", value );
	}
}
void InfoViewManager::setZlocation( float value ) {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
		m_pTranslation->put_Z( value );
		//log_messagef( "putting X location", value );
	}
}

void InfoViewManager::setRed( int ival ) {
    ISFColorObject* m_pDiffuseColor = getDiffuseColor();
	if ( m_pDiffuseColor == NULL ) {
		m_pDiffuseColor = getEmissiveColor();
		if ( m_pDiffuseColor == NULL ) {
			return;
		}
	}

    float r;

    if ( SUCCEEDED( m_pDiffuseColor->get_Red( &r ))) {
		r = (float)ival/(float)65535;
		m_pDiffuseColor -> put_Red ( r );
	}
}

void InfoViewManager::setGreen( int ival ) {
    ISFColorObject* m_pDiffuseColor = getDiffuseColor();
	if ( m_pDiffuseColor == NULL ) {
		m_pDiffuseColor = getEmissiveColor();
		if ( m_pDiffuseColor == NULL ) {
			return;
		}
	}

    float g;

    if ( SUCCEEDED( m_pDiffuseColor->get_Green( &g ))) {
		g = (float)ival/(float)65535;
		m_pDiffuseColor -> put_Green ( g );
	}
}

void InfoViewManager::setBlue( int ival ) {
    ISFColorObject* m_pDiffuseColor = getDiffuseColor();
	if ( m_pDiffuseColor == NULL ) {
		m_pDiffuseColor = getEmissiveColor();
		if ( m_pDiffuseColor == NULL ) {
			return;
		}
	}

    float b;

    if ( SUCCEEDED( m_pDiffuseColor->get_Blue( &b ))) {
		b = (float)ival/(float)65535;
		m_pDiffuseColor -> put_Blue ( b );
	}
}
void InfoViewManager::setRotation( int ival ) {
	//log_message("in setRotation" );
    ISFRotationObject* m_pDiffuseColor = rotationList[currentIdx];
	if ( m_pDiffuseColor == NULL ) {
		log_message( "no field found" );
		return;
	}

    float angle;

    if ( SUCCEEDED( m_pDiffuseColor->get_Angle( &angle ))) {
		//log_message( "Should have set it" );
		angle = ((float)ival/(float)65535)*(float)6.28;
		m_pDiffuseColor -> put_Angle( angle );
	}
}

void InfoViewManager::setIn1( int ival ) {
	//log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn1();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}

void InfoViewManager::setIn2( int ival ) {
	//log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn2();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}
void InfoViewManager::setIn3( int ival ) {
	//log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn3();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}
void InfoViewManager::setIn4( int ival ) {
	//log_messagei1( "InfoViewManager::setIn1", ival );
	ISFInt32Object* in1 = visualObjects[ currentIdx ]->getIn4();
	if ( in1 != NULL ) {
		log_message( "and actually calling method!" );
		in1->put_Value( ival );
	}
}

/**
 *  ======== InfoViewManager::SetValue ========
 *
 *  Purpose:
 *    Generic method for setting absolute values.  Which value gets
 *    set depends on last call to "SetProperty", which sets the
 *    "currentProperty" variable.
 *
 *  Input Parameters:
 *    valueStr -- the value string passed in.  Since this was designed
 *      for interoperability with "neattools", in most cases this will
 *      be a string form of a value from 0 to 65535.
 *
 *  Return Value:
 *    none
 */

void InfoViewManager::SetValue( char* valueStr ) {
	//log_message1( "SetValue: ", valueStr );
	if ( currentProperty == NULL ) {
		return;
	}
	if ( currentIdx < 0 ) {
		return;
	}
	int ival = atoi( valueStr );
	if ( strcmp( currentProperty, "transparency" ) == 0 ) {
		int integerPercent = ival/655;
		if (( integerPercent < 0 ) || ( integerPercent > 100 )) {
			integerPercent = 100;
		} 
		if ( gotMaterial[ currentIdx ] ) {
			ISFFloatObject* f = transparencyList[ currentIdx ];
			float value = 0.01f * integerPercent;
			if ( value > 1.0f ) {
				value = 1.0f;
			}
			f->put_Value(value);
		}
	} else if ( strcmp( currentProperty, "x" ) == 0 ) {
		int integerPercent = ival/655;
		float addition = (float)integerPercent * (xMax - xMin)/100;
		setXlocation( xMin + addition );
	} else if ( strcmp( currentProperty, "y" ) == 0 ) {
		int integerPercent = ival/655;
		float addition = (float)integerPercent * (yMax - yMin)/100;
		setYlocation( yMin + addition );
	} else if ( strcmp( currentProperty, "z" ) == 0 ) {
		int integerPercent = ival/655;
		float addition = (float)integerPercent * (zMax - zMin)/100;
		setZlocation( yMin + addition );
	} else if ( strcmp( currentProperty, "red" ) == 0 ) {
		setRed( ival );
	} else if ( strcmp( currentProperty, "green" ) == 0 ) {
		setGreen( ival );
	} else if ( strcmp( currentProperty, "blue" ) == 0 ) {
		setBlue( ival );
	} else if ( strcmp( currentProperty, "xscale" ) == 0 ) {
		setXscale( ival );
	} else if ( strcmp( currentProperty, "yscale" ) == 0 ) {
		setYscale( ival );
	} else if ( strcmp( currentProperty, "zscale" ) == 0 ) {
		setZscale( ival );
	} else if ( strcmp( currentProperty, "scale" ) == 0 ) {
		setScale( ival );
	} else if ( strcmp( currentProperty, "rotation" ) == 0 ) {
		setRotation( ival );
	} else if ( strcmp( currentProperty, "in1" ) == 0 ) {
		setIn1( ival );
	} else if ( strcmp( currentProperty, "in2" ) == 0 ) {
		setIn2( ival );
	} else if ( strcmp( currentProperty, "in3" ) == 0 ) {
		setIn3( ival );
	} else if ( strcmp( currentProperty, "in4" ) == 0 ) {
		setIn4( ival );
	}
}


//
//  Show and hide, this should be re-implemented via a Switch node and
//  this should then send values to a Script node that knows how to
//  switch any of its child nodes on and off.
//

void InfoViewManager::Show() {
	if (( currentIdx >= 0 ) && !visible[currentIdx] ) {
		scaleObject( 10000 );
		visible[currentIdx] = true;
	}
}
void InfoViewManager::Hide() {
	if (( currentIdx >= 0 ) && visible[currentIdx] ) {
		scaleObject( 1 );
		visible[currentIdx] = false;
	}
}



int InfoViewManager::getIdx( char* name ) {
	//log_message1( "try to find: ", name );
	for ( int i = 0; i < transformIdx; i++ ) {
		//log_message1( "compare to: ", transformNameList[i] );
		if ( strcmp( name, transformNameList[i] ) == 0 ) {
			currentIdx = i;
			return( i );
		}
	}
	return( -1 );
}

void InfoViewManager::MoveUp() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
		float y;
		if ( SUCCEEDED ( m_pTranslation -> get_Y ( &y ) ) )	{
			y += .2f; // move it up on 20 cm
			m_pTranslation -> put_Y ( y );
		}
	}
}

void InfoViewManager::MoveDown() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
		float y;
		if ( SUCCEEDED ( m_pTranslation -> get_Y ( &y ) ) )	{
			y -= .2f; // move it up on 20 cm
			m_pTranslation -> put_Y ( y );
		}
	}
}

void InfoViewManager::MoveRight() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float x;

    if ( SUCCEEDED ( m_pTranslation -> get_X ( &x ) ) )
    {
        x += .2f; // move it right on 20 cm
        m_pTranslation -> put_X ( x );
    }
	}
}

void InfoViewManager::MoveLeft() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float x;

    if ( SUCCEEDED ( m_pTranslation -> get_X ( &x ) ) )
    {
        x -= .2f; // move it right on 20 cm
        m_pTranslation -> put_X ( x );
    }
	}
}

void InfoViewManager::MoveIn() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float z;

		if ( SUCCEEDED ( m_pTranslation -> get_Z ( &z ) ) )    {
			z += .2f; // move it forward on 20 cm
			m_pTranslation -> put_Z ( z );
		}
	}
}

void InfoViewManager::MoveOut() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float z;

		if ( SUCCEEDED ( m_pTranslation -> get_Z ( &z ) ) )    {
			z -= .2f; // move it forward on 20 cm
			m_pTranslation -> put_Z ( z );
		}
	}
}



char* InfoViewManager::getObjectAt( int offset ) {
	if ( offset < transformIdx ) {
		return( transformNameList[ offset ] );
	} else {
		return( "" );
	}
}

bool InfoViewManager::isVisible( int offset ) {
	if ( offset < transformIdx ) {
		return( visible[ offset ] );
	} else {
		return( false );
	}
}


/**
 * ======== InfoViewManager::SetXmin, SetYmin, SetZmin ========
 *
 *  Purpose:
 *    Set the floating point value for the minimum X/Y/Z location.  Used for
 *    mapping 0-65535 values (from neattools) into actual X/Y/Z locations.
 *
 *  Input Parameters:
 *    valueStr -- string form of minimum float value
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetXmin( char* valueStr ) {
	xMin = (float)atof( valueStr );
	log_messagef( "XMIN now set", xMin );
}
void InfoViewManager::SetYmin( char* valueStr ) {
	yMin = (float)atof( valueStr );
	log_messagef( "YMIN now set", yMin );
}
void InfoViewManager::SetZmin( char* valueStr ) {
	zMin = (float)atof( valueStr );
	log_messagef( "ZMIN now set", zMin );
}


/**
 * ======== InfoViewManager::SetXmax ========
 *
 *  Purpose:
 *    Set the floating point value for the maximum X location.  Used for
 *    mapping 0-65535 values (from neattools) into actual X locations.
 *
 *  Input Parameters:
 *    valueStr -- string form of maximum float value
 *
 *  Return Value:
 *    none.
 */

void InfoViewManager::SetXmax( char* valueStr ) {
	xMax = (float)atof( valueStr );
	log_messagef( "XMAX now set", xMax );
}
void InfoViewManager::SetYmax( char* valueStr ) {
	yMax = (float)atof( valueStr );
	log_messagef( "YMAX now set", yMax );
}
void InfoViewManager::SetZmax( char* valueStr ) {
	zMax = (float)atof( valueStr );
	log_messagef( "ZMAX now set", zMax );
}


/**
 * ======== InfoViewManager::rescale ========
 *
 *  Purpose:  
 *    rescale a floating point value, handling decreases in scale
 *    by converting percent to a reversable value.  For example,
 *    increasing size by 5 percent requires a corresponding decrease
 *    in value that goes back from 105% size to 100% size.  This means
 *    a less than 100 percent value coming in gets transferred from
 *    NN/100 to (NN+X=100)/100+X.
 */

float InfoViewManager::rescale( float value, int percent ) {
	float percentAsFloat = (float)(percent)/100;
	if ( percent < 100 ) {
		int x = 100 - percent;
		percentAsFloat = (float)(100)/(float)(100+x);
	}
	return( value * percentAsFloat );
}


void InfoViewManager::setXscale( int neattools_value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float x;

			if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) ) {
				x = (float)neattools_value/(float)(65535/3);
				m_pScale->put_X( x );
			}
		}
	}
}
void InfoViewManager::setYscale( int neattools_value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float y;

			if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) ) {
				y = (float)neattools_value/(float)(65535/3);
				m_pScale->put_Y( y );
			}
		}
	}
}
void InfoViewManager::setZscale( int neattools_value ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float z;

			if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) ) {
				z = (float)neattools_value/(float)(65535/3);
				m_pScale->put_Z( z );
			}
		}
	}
}
void InfoViewManager::setScale( int neattools_value ) {
	setXscale( neattools_value );
	setYscale( neattools_value );
	setZscale( neattools_value );
}


void InfoViewManager::scaleObjectX( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float x;

			if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) )	{
				m_pScale -> put_X( rescale( x, percent ));
			}
		}
	}
}

void InfoViewManager::scaleObjectY( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float y;

			if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) )	{
				m_pScale -> put_Y( rescale( y, percent ));
			}
		}
	}
}

void InfoViewManager::scaleObjectZ( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale != NULL ) {
			float z;

			if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) )	{
				m_pScale -> put_Z( rescale( z, percent ));
			}
		}
	}
}

void InfoViewManager::scaleObject( int percent ) {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFVec3fObject* m_pScale = vo->getScale();
		if ( m_pScale == NULL ) {
			log_message( "HEY, m_pScale is NULL" );
		} else {
			//log_message( "Seem to have got an m_pScale" );
			float x;
			float y;
			float z;

			if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) )	{
				m_pScale -> put_X( rescale( x, percent ));
			}
			if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) )	{
				y = y * (float)(percent)/100; // move it up on 20 cm
				m_pScale -> put_Y( rescale( y, percent ));
			}
			if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) )	{
				m_pScale -> put_Z( rescale( z, percent ));
			}
		}
	}
}

void InfoViewManager::setTransparency( ISFFloatObject* t ) {
	if ( t == NULL ) {
		log_message( "NULL transparency" );
	} else {
		//log_messagei1( "setTransparency, idx", currentIdx );
	}
	transparencyList[ currentIdx ] = t; 
	gotMaterial[ currentIdx ] = true;
}


void InfoViewManager::RotateX() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 1 );
			r->put_Y( 0 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle += (float).157;
				if ( angle > (float)6.28 ) {
					angle -= (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}

void InfoViewManager::RotateY() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 1 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle += (float).157;
				if ( angle > (float)6.28 ) {
					angle -= (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}
void InfoViewManager::RotateZ() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 0 );
			r->put_Z( 1 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle += (float).157;
				if ( angle > (float)6.28 ) {
					angle -= (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}
void InfoViewManager::RotateXcc() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 1 );
			r->put_Y( 0 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle -= (float).157;
				if ( angle < (float)0 ) {
					angle += (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}
void InfoViewManager::RotateYcc() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 1 );
			r->put_Z( 0 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle -= (float).157;
				if ( angle < (float)0 ) {
					angle += (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}
void InfoViewManager::RotateZcc() {
	if ( currentIdx >= 0 ) {
		VisualObject* vo = visualObjects[currentIdx];
		ISFRotationObject* r = vo->getRotation();
		if ( r != NULL ) {
			r->put_X( 0 );
			r->put_Y( 0 );
			r->put_Z( 1 );
			float angle;

			if ( SUCCEEDED( r->get_Angle( &angle ))) {
				//log_message( "Should have set it" );
				angle -= (float).157;
				if ( angle < (float)0 ) {
					angle += (float)6.28;
				}
				r -> put_Angle( angle );
			}
		}
	}
}


//
//  Track objects added to renderer.
//
//  Assumes global variable "transformName" has name of object being added.
//

extern char* transformName;  // should not be a global, but is ... needs to move into InfoViewManager

void InfoViewManager::addObject( CCubesView* original, INodeObject* o, VisualObject* vo ) {
	ISFColorObject* pDiffuseColor = NULL;
	ISFColorObject* pEmissiveColor = NULL;
	ISFVec3fObject* pTranslation = NULL;
	ISFRotationObject* pRotation = NULL;
	ISFVec3fObject* pScale = NULL;
	ISFFloatObject* pTransparency = NULL;
	//log_message( "getting translation" );
	log_message1( "Current object:", transformName );
	//log_messagei1( "idx is ", transformIdx );
	if ( !original->GetTranslationField( o, &pTranslation )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load translation field" );
		//exit( 0 );
	}
	if ( !original->GetRotationField( o, &pRotation )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load rotation field" );
		//exit( 0 );
	}
	//log_message( "getting scale" );
	if ( !original->GetScaleField( o, &pScale )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load scale field" );
		//exit( 0 );
	}
	//log_message( "getting diffuseColor" );
	gotMaterial[ transformIdx ] = true;
    if ( !original->GetDiffuseColorField( o, &pDiffuseColor, transformName )) {
		//log_error1( "Object: ", transformName );
		//log_error( "..has no diffuseColor field" );
		gotMaterial[ transformIdx ] = false;
		//exit( 0 );
	}
	//log_message( "getting emissiveColor" );
    if ( !original->GetEmissiveColorField( o, &pEmissiveColor, transformName )) {
		//log_error1( "Object: ", transformName );
		//log_error( "..has no emissiveColor field" );
		gotMaterial[ transformIdx ] = false;
		//exit( 0 );
	}
	//log_message( "getting transparency" );
	if ( !original->GetTransparencyField( o, &pTransparency, transformName )) {
		log_error1( "Object: ", transformName );
		log_error( "..has no transparency field" );
		gotMaterial[ transformIdx ] = false;
		//exit( 0 );
	}
	//log_message( "done extracting attributes" );
	locationList[transformIdx ] = pTranslation;
	scaleList[transformIdx] = pScale;
	colorList[ transformIdx ] = pDiffuseColor;
	ecolorList[ transformIdx ] = pEmissiveColor;
	transparencyList[ transformIdx ] = pTransparency;
	rotationList[ transformIdx ] = pRotation;
	transformNameList[ transformIdx ] = newstrcpy(transformName);
	visible[ transformIdx ] = true;
	value[ transformIdx ] = 0;
	visualObjects[ transformIdx ] = vo;
	currentIdx = transformIdx;
	transformIdx++;
}
